# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class ActivityOnInactiveMarkAs(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status result of the activity when the state is set to Inactive. This is an optional property
    and if not provided when the activity is inactive, the status will be Succeeded by default.
    """

    SUCCEEDED = "Succeeded"
    """Succeeded."""
    FAILED = "Failed"
    """Failed."""
    SKIPPED = "Skipped"
    """Skipped."""


class ActivityState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Activity state. This is an optional property and if not provided, the state will be Active by
    default.
    """

    ACTIVE = "Active"
    """Active."""
    INACTIVE = "Inactive"
    """Inactive."""


class AmazonRdsForOracleAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Authentication type for connecting to the AmazonRdsForOracle database. Only used for Version
    2.0.
    """

    BASIC = "Basic"
    """Basic."""


class AmazonRdsForSqlAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    SQL = "SQL"
    """SQL."""
    WINDOWS = "Windows"
    """Windows."""


class AzureFunctionActivityMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The list of HTTP methods supported by a AzureFunctionActivity."""

    GET = "GET"
    """GET."""
    POST = "POST"
    """POST."""
    PUT = "PUT"
    """PUT."""
    DELETE = "DELETE"
    """DELETE."""
    OPTIONS = "OPTIONS"
    """OPTIONS."""
    HEAD = "HEAD"
    """HEAD."""
    TRACE = "TRACE"
    """TRACE."""


class AzurePostgreSqlWriteMethodEnum(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The write behavior for the operation. Default is Bulk Insert."""

    BULK_INSERT = "BulkInsert"
    """BulkInsert."""
    COPY_COMMAND = "CopyCommand"
    """CopyCommand."""
    UPSERT = "Upsert"
    """Upsert."""


class AzureSearchIndexWriteBehaviorType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify the write behavior when upserting documents into Azure Search Index."""

    MERGE = "Merge"
    """Merge."""
    UPLOAD = "Upload"
    """Upload."""


class AzureSqlDatabaseAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    SQL = "SQL"
    """SQL."""
    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class AzureSqlDWAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    SQL = "SQL"
    """SQL."""
    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class AzureSqlMIAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    SQL = "SQL"
    """SQL."""
    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class AzureStorageAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    ACCOUNT_KEY = "AccountKey"
    """AccountKey."""
    SAS_URI = "SasUri"
    """SasUri."""
    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    MSI = "Msi"
    """Msi."""


class BigDataPoolReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Big data pool reference type."""

    BIG_DATA_POOL_REFERENCE = "BigDataPoolReference"
    """BigDataPoolReference."""


class BlobEventTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of BlobEventTypes."""

    MICROSOFT_STORAGE_BLOB_CREATED = "Microsoft.Storage.BlobCreated"
    """Microsoft.Storage.BlobCreated."""
    MICROSOFT_STORAGE_BLOB_DELETED = "Microsoft.Storage.BlobDeleted"
    """Microsoft.Storage.BlobDeleted."""


class CassandraSourceReadConsistencyLevels(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The consistency level specifies how many Cassandra servers must respond to a read request
    before returning data to the client application. Cassandra checks the specified number of
    Cassandra servers for data to satisfy the read request. Must be one of
    cassandraSourceReadConsistencyLevels. The default value is 'ONE'. It is case-insensitive.
    """

    ALL = "ALL"
    """ALL."""
    EACH_QUORUM = "EACH_QUORUM"
    """EACH_QUORUM."""
    QUORUM = "QUORUM"
    """QUORUM."""
    LOCAL_QUORUM = "LOCAL_QUORUM"
    """LOCAL_QUORUM."""
    ONE = "ONE"
    """ONE."""
    TWO = "TWO"
    """TWO."""
    THREE = "THREE"
    """THREE."""
    LOCAL_ONE = "LOCAL_ONE"
    """LOCAL_ONE."""
    SERIAL = "SERIAL"
    """SERIAL."""
    LOCAL_SERIAL = "LOCAL_SERIAL"
    """LOCAL_SERIAL."""


class ConfigurationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the spark config."""

    DEFAULT = "Default"
    """Default."""
    CUSTOMIZED = "Customized"
    """Customized."""
    ARTIFACT = "Artifact"
    """Artifact."""


class ConnectionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of connection via linked service or dataset."""

    LINKEDSERVICETYPE = "linkedservicetype"
    """linkedservicetype."""


class CosmosDbConnectionMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The connection mode used to access CosmosDB account. Type: string."""

    GATEWAY = "Gateway"
    """Gateway."""
    DIRECT = "Direct"
    """Direct."""


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of entity that created the resource."""

    USER = "User"
    """The entity was created by a user."""
    APPLICATION = "Application"
    """The entity was created by an application."""
    MANAGED_IDENTITY = "ManagedIdentity"
    """The entity was created by a managed identity."""
    KEY = "Key"
    """The entity was created by a key."""


class CredentialReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Credential reference type."""

    CREDENTIAL_REFERENCE = "CredentialReference"
    """CredentialReference."""


class DataFlowComputeType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Compute type of the cluster which will execute data flow job."""

    GENERAL = "General"
    """General."""
    MEMORY_OPTIMIZED = "MemoryOptimized"
    """MemoryOptimized."""
    COMPUTE_OPTIMIZED = "ComputeOptimized"
    """ComputeOptimized."""


class DataFlowDebugCommandType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The command type."""

    EXECUTE_PREVIEW_QUERY = "executePreviewQuery"
    """executePreviewQuery."""
    EXECUTE_STATISTICS_QUERY = "executeStatisticsQuery"
    """executeStatisticsQuery."""
    EXECUTE_EXPRESSION_QUERY = "executeExpressionQuery"
    """executeExpressionQuery."""


class DataFlowReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Data flow reference type."""

    DATA_FLOW_REFERENCE = "DataFlowReference"
    """DataFlowReference."""


class DatasetReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Dataset reference type."""

    DATASET_REFERENCE = "DatasetReference"
    """DatasetReference."""


class DayOfWeek(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The days of the week."""

    SUNDAY = "Sunday"
    """Sunday."""
    MONDAY = "Monday"
    """Monday."""
    TUESDAY = "Tuesday"
    """Tuesday."""
    WEDNESDAY = "Wednesday"
    """Wednesday."""
    THURSDAY = "Thursday"
    """Thursday."""
    FRIDAY = "Friday"
    """Friday."""
    SATURDAY = "Saturday"
    """Saturday."""


class DaysOfWeek(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of DaysOfWeek."""

    SUNDAY = "Sunday"
    """Sunday."""
    MONDAY = "Monday"
    """Monday."""
    TUESDAY = "Tuesday"
    """Tuesday."""
    WEDNESDAY = "Wednesday"
    """Wednesday."""
    THURSDAY = "Thursday"
    """Thursday."""
    FRIDAY = "Friday"
    """Friday."""
    SATURDAY = "Saturday"
    """Saturday."""


class Db2AuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AuthenticationType to be used for connection. It is mutually exclusive with connectionString
    property.
    """

    BASIC = "Basic"
    """Basic."""


class DependencyCondition(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of DependencyCondition."""

    SUCCEEDED = "Succeeded"
    """Succeeded."""
    FAILED = "Failed"
    """Failed."""
    SKIPPED = "Skipped"
    """Skipped."""
    COMPLETED = "Completed"
    """Completed."""


class DynamicsSinkWriteBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Defines values for DynamicsSinkWriteBehavior."""

    UPSERT = "Upsert"
    """Upsert."""


class EventSubscriptionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Event Subscription Status."""

    ENABLED = "Enabled"
    """Enabled."""
    PROVISIONING = "Provisioning"
    """Provisioning."""
    DEPROVISIONING = "Deprovisioning"
    """Deprovisioning."""
    DISABLED = "Disabled"
    """Disabled."""
    UNKNOWN = "Unknown"
    """Unknown."""


class ExpressionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Expression type."""

    EXPRESSION = "Expression"
    """Expression."""


class ExpressionV2Type(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of expressions supported by the system. Type: string."""

    CONSTANT = "Constant"
    """Constant."""
    FIELD = "Field"
    """Field."""
    UNARY = "Unary"
    """Unary."""
    BINARY = "Binary"
    """Binary."""
    N_ARY = "NAry"
    """NAry."""


class FactoryIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The identity type."""

    SYSTEM_ASSIGNED = "SystemAssigned"
    """SystemAssigned."""
    USER_ASSIGNED = "UserAssigned"
    """UserAssigned."""
    SYSTEM_ASSIGNED_USER_ASSIGNED = "SystemAssigned,UserAssigned"
    """SystemAssigned,UserAssigned."""


class FrequencyType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Frequency of period in terms of 'Hour', 'Minute' or 'Second'."""

    HOUR = "Hour"
    """Hour."""
    MINUTE = "Minute"
    """Minute."""
    SECOND = "Second"
    """Second."""


class FtpAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to be used to connect to the FTP server."""

    BASIC = "Basic"
    """Basic."""
    ANONYMOUS = "Anonymous"
    """Anonymous."""


class GlobalParameterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Global Parameter type."""

    OBJECT = "Object"
    """Object."""
    STRING = "String"
    """String."""
    INT = "Int"
    """Int."""
    FLOAT = "Float"
    """Float."""
    BOOL = "Bool"
    """Bool."""
    ARRAY = "Array"
    """Array."""


class GoogleAdWordsAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The OAuth 2.0 authentication mechanism used for authentication. ServiceAuthentication can only
    be used on self-hosted IR.
    """

    SERVICE_AUTHENTICATION = "ServiceAuthentication"
    """ServiceAuthentication."""
    USER_AUTHENTICATION = "UserAuthentication"
    """UserAuthentication."""


class GoogleBigQueryAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The OAuth 2.0 authentication mechanism used for authentication. ServiceAuthentication can only
    be used on self-hosted IR.
    """

    SERVICE_AUTHENTICATION = "ServiceAuthentication"
    """ServiceAuthentication."""
    USER_AUTHENTICATION = "UserAuthentication"
    """UserAuthentication."""


class GoogleBigQueryV2AuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The OAuth 2.0 authentication mechanism used for authentication."""

    SERVICE_AUTHENTICATION = "ServiceAuthentication"
    """ServiceAuthentication."""
    USER_AUTHENTICATION = "UserAuthentication"
    """UserAuthentication."""


class GreenplumAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use. Type: string. Only used for V2."""

    BASIC = "Basic"
    """Basic."""


class HBaseAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication mechanism to use to connect to the HBase server."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    BASIC = "Basic"
    """Basic."""


class HDInsightActivityDebugInfoOption(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The HDInsightActivityDebugInfoOption settings to use."""

    NONE = "None"
    """None."""
    ALWAYS = "Always"
    """Always."""
    FAILURE = "Failure"
    """Failure."""


class HDInsightClusterAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """HDInsight cluster authentication type."""

    BASIC_AUTH = "BasicAuth"
    """BasicAuth."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class HDInsightOndemandClusterResourceGroupAuthenticationType(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """HDInsight On-demand cluster resource group authentication type."""

    SERVICE_PRINCIPAL_KEY = "ServicePrincipalKey"
    """ServicePrincipalKey."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class HiveAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication method used to access the Hive server."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    USERNAME = "Username"
    """Username."""
    USERNAME_AND_PASSWORD = "UsernameAndPassword"
    """UsernameAndPassword."""
    WINDOWS_AZURE_HD_INSIGHT_SERVICE = "WindowsAzureHDInsightService"
    """WindowsAzureHDInsightService."""


class HiveServerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of Hive server."""

    HIVE_SERVER1 = "HiveServer1"
    """HiveServer1."""
    HIVE_SERVER2 = "HiveServer2"
    """HiveServer2."""
    HIVE_THRIFT_SERVER = "HiveThriftServer"
    """HiveThriftServer."""


class HiveThriftTransportProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The transport protocol to use in the Thrift layer."""

    BINARY = "Binary"
    """Binary."""
    SASL = "SASL"
    """SASL."""
    HTTP = "HTTP "
    """HTTP."""


class HttpAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to be used to connect to the HTTP server."""

    BASIC = "Basic"
    """Basic."""
    ANONYMOUS = "Anonymous"
    """Anonymous."""
    DIGEST = "Digest"
    """Digest."""
    WINDOWS = "Windows"
    """Windows."""
    CLIENT_CERTIFICATE = "ClientCertificate"
    """ClientCertificate."""


class ImpalaAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    SASL_USERNAME = "SASLUsername"
    """SASLUsername."""
    USERNAME_AND_PASSWORD = "UsernameAndPassword"
    """UsernameAndPassword."""


class ImpalaThriftTransportProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The transport protocol to use in the Thrift layer (for V2 only). Default value is Binary."""

    BINARY = "Binary"
    """Binary."""
    HTTP = "HTTP"
    """HTTP."""


class IntegrationRuntimeAuthKeyName(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The name of the authentication key to regenerate."""

    AUTH_KEY1 = "authKey1"
    """authKey1."""
    AUTH_KEY2 = "authKey2"
    """authKey2."""


class IntegrationRuntimeAutoUpdate(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The state of integration runtime auto update."""

    ON = "On"
    """On."""
    OFF = "Off"
    """Off."""


class IntegrationRuntimeEdition(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The edition for the SSIS Integration Runtime."""

    STANDARD = "Standard"
    """Standard."""
    ENTERPRISE = "Enterprise"
    """Enterprise."""


class IntegrationRuntimeEntityReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of this referenced entity."""

    INTEGRATION_RUNTIME_REFERENCE = "IntegrationRuntimeReference"
    """IntegrationRuntimeReference."""
    LINKED_SERVICE_REFERENCE = "LinkedServiceReference"
    """LinkedServiceReference."""


class IntegrationRuntimeInternalChannelEncryptionMode(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """It is used to set the encryption mode for node-node communication channel (when more than 2
    self-hosted integration runtime nodes exist).
    """

    NOT_SET = "NotSet"
    """NotSet."""
    SSL_ENCRYPTED = "SslEncrypted"
    """SslEncrypted."""
    NOT_ENCRYPTED = "NotEncrypted"
    """NotEncrypted."""


class IntegrationRuntimeLicenseType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """License type for bringing your own license scenario."""

    BASE_PRICE = "BasePrice"
    """BasePrice."""
    LICENSE_INCLUDED = "LicenseIncluded"
    """LicenseIncluded."""


class IntegrationRuntimeReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of integration runtime."""

    INTEGRATION_RUNTIME_REFERENCE = "IntegrationRuntimeReference"
    """IntegrationRuntimeReference."""


class IntegrationRuntimeSsisCatalogPricingTier(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The pricing tier for the catalog database. The valid values could be found in
    `https://azure.microsoft.com/en-us/pricing/details/sql-database/
    <https://azure.microsoft.com/en-us/pricing/details/sql-database/>`_.
    """

    BASIC = "Basic"
    """Basic."""
    STANDARD = "Standard"
    """Standard."""
    PREMIUM = "Premium"
    """Premium."""
    PREMIUM_RS = "PremiumRS"
    """PremiumRS."""


class IntegrationRuntimeState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The state of integration runtime."""

    INITIAL = "Initial"
    """Initial."""
    STOPPED = "Stopped"
    """Stopped."""
    STARTED = "Started"
    """Started."""
    STARTING = "Starting"
    """Starting."""
    STOPPING = "Stopping"
    """Stopping."""
    NEED_REGISTRATION = "NeedRegistration"
    """NeedRegistration."""
    ONLINE = "Online"
    """Online."""
    LIMITED = "Limited"
    """Limited."""
    OFFLINE = "Offline"
    """Offline."""
    ACCESS_DENIED = "AccessDenied"
    """AccessDenied."""


class IntegrationRuntimeType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of integration runtime."""

    MANAGED = "Managed"
    """Managed."""
    SELF_HOSTED = "SelfHosted"
    """SelfHosted."""


class IntegrationRuntimeUpdateResult(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The result of the last integration runtime node update."""

    NONE = "None"
    """None."""
    SUCCEED = "Succeed"
    """Succeed."""
    FAIL = "Fail"
    """Fail."""


class InteractiveCapabilityStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The interactive authoring capability status. Must be one of InteractiveCapabilityStatus. The
    default value is 'Enabling'.
    """

    ENABLING = "Enabling"
    """Enabling."""
    ENABLED = "Enabled"
    """Enabled."""
    DISABLING = "Disabling"
    """Disabling."""
    DISABLED = "Disabled"
    """Disabled."""


class LakehouseAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class ManagedIntegrationRuntimeNodeStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The managed integration runtime node status."""

    STARTING = "Starting"
    """Starting."""
    AVAILABLE = "Available"
    """Available."""
    RECYCLING = "Recycling"
    """Recycling."""
    UNAVAILABLE = "Unavailable"
    """Unavailable."""


class ManagedVirtualNetworkReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Managed Virtual Network reference type."""

    MANAGED_VIRTUAL_NETWORK_REFERENCE = "ManagedVirtualNetworkReference"
    """ManagedVirtualNetworkReference."""


class MappingType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the CDC attribute mapping. Note: 'Advanced' mapping type is also saved as 'Derived'."""

    DIRECT = "Direct"
    """Direct."""
    DERIVED = "Derived"
    """Derived."""
    AGGREGATE = "Aggregate"
    """Aggregate."""


class MongoDbAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to be used to connect to the MongoDB database."""

    BASIC = "Basic"
    """Basic."""
    ANONYMOUS = "Anonymous"
    """Anonymous."""


class NetezzaSecurityLevelType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specifies the security level for the driver connection to the data store. PreferredUnSecured :
    prefer unsecured, allow fallback to secured connection if required. OnlyUnSecured : strictly
    unsecured, no fallback.
    """

    PREFERRED_UN_SECURED = "PreferredUnSecured"
    """PreferredUnSecured."""
    ONLY_UN_SECURED = "OnlyUnSecured"
    """OnlyUnSecured."""


class NotebookParameterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Notebook parameter type."""

    STRING = "string"
    """string."""
    INT = "int"
    """int."""
    FLOAT = "float"
    """float."""
    BOOL = "bool"
    """bool."""


class NotebookReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Synapse notebook reference type."""

    NOTEBOOK_REFERENCE = "NotebookReference"
    """NotebookReference."""


class ODataAadServicePrincipalCredentialType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Specify the credential type (key or cert) is used for service principal."""

    SERVICE_PRINCIPAL_KEY = "ServicePrincipalKey"
    """ServicePrincipalKey."""
    SERVICE_PRINCIPAL_CERT = "ServicePrincipalCert"
    """ServicePrincipalCert."""


class ODataAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of authentication used to connect to the OData service."""

    BASIC = "Basic"
    """Basic."""
    ANONYMOUS = "Anonymous"
    """Anonymous."""
    WINDOWS = "Windows"
    """Windows."""
    AAD_SERVICE_PRINCIPAL = "AadServicePrincipal"
    """AadServicePrincipal."""
    MANAGED_SERVICE_IDENTITY = "ManagedServiceIdentity"
    """ManagedServiceIdentity."""


class OracleAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Authentication type for connecting to the Oracle database. Only used for Version 2.0."""

    BASIC = "Basic"
    """Basic."""


class ParameterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Parameter type."""

    OBJECT = "Object"
    """Object."""
    STRING = "String"
    """String."""
    INT = "Int"
    """Int."""
    FLOAT = "Float"
    """Float."""
    BOOL = "Bool"
    """Bool."""
    ARRAY = "Array"
    """Array."""
    SECURE_STRING = "SecureString"
    """SecureString."""


class PhoenixAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication mechanism used to connect to the Phoenix server."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    USERNAME_AND_PASSWORD = "UsernameAndPassword"
    """UsernameAndPassword."""
    WINDOWS_AZURE_HD_INSIGHT_SERVICE = "WindowsAzureHDInsightService"
    """WindowsAzureHDInsightService."""


class PipelineReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Pipeline reference type."""

    PIPELINE_REFERENCE = "PipelineReference"
    """PipelineReference."""


class PolybaseSettingsRejectType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Indicates whether the RejectValue property is specified as a literal value or a percentage."""

    VALUE = "value"
    """value."""
    PERCENTAGE = "percentage"
    """percentage."""


class PrestoAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication mechanism used to connect to the Presto server."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    LDAP = "LDAP"
    """LDAP."""


class PublicNetworkAccess(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Whether or not public network access is allowed for the data factory."""

    ENABLED = "Enabled"
    """Enabled."""
    DISABLED = "Disabled"
    """Disabled."""


class RecurrenceFrequency(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumerates possible frequency option for the schedule trigger."""

    NOT_SPECIFIED = "NotSpecified"
    """NotSpecified."""
    MINUTE = "Minute"
    """Minute."""
    HOUR = "Hour"
    """Hour."""
    DAY = "Day"
    """Day."""
    WEEK = "Week"
    """Week."""
    MONTH = "Month"
    """Month."""
    YEAR = "Year"
    """Year."""


class RestServiceAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of authentication used to connect to the REST service."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    BASIC = "Basic"
    """Basic."""
    AAD_SERVICE_PRINCIPAL = "AadServicePrincipal"
    """AadServicePrincipal."""
    MANAGED_SERVICE_IDENTITY = "ManagedServiceIdentity"
    """ManagedServiceIdentity."""
    O_AUTH2_CLIENT_CREDENTIAL = "OAuth2ClientCredential"
    """OAuth2ClientCredential."""


class RunQueryFilterOperand(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Parameter name to be used for filter. The allowed operands to query pipeline runs are
    PipelineName, RunStart, RunEnd and Status; to query activity runs are ActivityName,
    ActivityRunStart, ActivityRunEnd, ActivityType and Status, and to query trigger runs are
    TriggerName, TriggerRunTimestamp and Status.
    """

    PIPELINE_NAME = "PipelineName"
    """PipelineName."""
    STATUS = "Status"
    """Status."""
    RUN_START = "RunStart"
    """RunStart."""
    RUN_END = "RunEnd"
    """RunEnd."""
    ACTIVITY_NAME = "ActivityName"
    """ActivityName."""
    ACTIVITY_RUN_START = "ActivityRunStart"
    """ActivityRunStart."""
    ACTIVITY_RUN_END = "ActivityRunEnd"
    """ActivityRunEnd."""
    ACTIVITY_TYPE = "ActivityType"
    """ActivityType."""
    TRIGGER_NAME = "TriggerName"
    """TriggerName."""
    TRIGGER_RUN_TIMESTAMP = "TriggerRunTimestamp"
    """TriggerRunTimestamp."""
    RUN_GROUP_ID = "RunGroupId"
    """RunGroupId."""
    LATEST_ONLY = "LatestOnly"
    """LatestOnly."""


class RunQueryFilterOperator(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Operator to be used for filter."""

    EQUALS = "Equals"
    """Equals."""
    NOT_EQUALS = "NotEquals"
    """NotEquals."""
    IN = "In"
    """In."""
    NOT_IN = "NotIn"
    """NotIn."""


class RunQueryOrder(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Sorting order of the parameter."""

    ASC = "ASC"
    """ASC."""
    DESC = "DESC"
    """DESC."""


class RunQueryOrderByField(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Parameter name to be used for order by. The allowed parameters to order by for pipeline runs
    are PipelineName, RunStart, RunEnd and Status; for activity runs are ActivityName,
    ActivityRunStart, ActivityRunEnd and Status; for trigger runs are TriggerName,
    TriggerRunTimestamp and Status.
    """

    RUN_START = "RunStart"
    """RunStart."""
    RUN_END = "RunEnd"
    """RunEnd."""
    PIPELINE_NAME = "PipelineName"
    """PipelineName."""
    STATUS = "Status"
    """Status."""
    ACTIVITY_NAME = "ActivityName"
    """ActivityName."""
    ACTIVITY_RUN_START = "ActivityRunStart"
    """ActivityRunStart."""
    ACTIVITY_RUN_END = "ActivityRunEnd"
    """ActivityRunEnd."""
    TRIGGER_NAME = "TriggerName"
    """TriggerName."""
    TRIGGER_RUN_TIMESTAMP = "TriggerRunTimestamp"
    """TriggerRunTimestamp."""


class SalesforceSinkWriteBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The write behavior for the operation. Default is Insert."""

    INSERT = "Insert"
    """Insert."""
    UPSERT = "Upsert"
    """Upsert."""


class SalesforceV2SinkWriteBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The write behavior for the operation. Default is Insert."""

    INSERT = "Insert"
    """Insert."""
    UPSERT = "Upsert"
    """Upsert."""


class SapCloudForCustomerSinkWriteBehavior(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The write behavior for the operation. Default is 'Insert'."""

    INSERT = "Insert"
    """Insert."""
    UPDATE = "Update"
    """Update."""


class SapHanaAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to be used to connect to the SAP HANA server."""

    BASIC = "Basic"
    """Basic."""
    WINDOWS = "Windows"
    """Windows."""


class ScriptActivityLogDestination(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The destination of logs. Type: string."""

    ACTIVITY_OUTPUT = "ActivityOutput"
    """ActivityOutput."""
    EXTERNAL_STORE = "ExternalStore"
    """ExternalStore."""


class ScriptActivityParameterDirection(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The direction of the parameter."""

    INPUT = "Input"
    """Input."""
    OUTPUT = "Output"
    """Output."""
    INPUT_OUTPUT = "InputOutput"
    """InputOutput."""


class ScriptActivityParameterType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the parameter."""

    BOOLEAN = "Boolean"
    """Boolean."""
    DATE_TIME = "DateTime"
    """DateTime."""
    DATE_TIME_OFFSET = "DateTimeOffset"
    """DateTimeOffset."""
    DECIMAL = "Decimal"
    """Decimal."""
    DOUBLE = "Double"
    """Double."""
    GUID = "Guid"
    """Guid."""
    INT16 = "Int16"
    """Int16."""
    INT32 = "Int32"
    """Int32."""
    INT64 = "Int64"
    """Int64."""
    SINGLE = "Single"
    """Single."""
    STRING = "String"
    """String."""
    TIMESPAN = "Timespan"
    """Timespan."""


class SelfHostedIntegrationRuntimeNodeStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Status of the integration runtime node."""

    NEED_REGISTRATION = "NeedRegistration"
    """NeedRegistration."""
    ONLINE = "Online"
    """Online."""
    LIMITED = "Limited"
    """Limited."""
    OFFLINE = "Offline"
    """Offline."""
    UPGRADING = "Upgrading"
    """Upgrading."""
    INITIALIZING = "Initializing"
    """Initializing."""
    INITIALIZE_FAILED = "InitializeFailed"
    """InitializeFailed."""


class ServiceNowAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    BASIC = "Basic"
    """Basic."""
    O_AUTH2 = "OAuth2"
    """OAuth2."""


class ServiceNowV2AuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    BASIC = "Basic"
    """Basic."""
    O_AUTH2 = "OAuth2"
    """OAuth2."""


class SftpAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to be used to connect to the FTP server."""

    BASIC = "Basic"
    """Basic."""
    SSH_PUBLIC_KEY = "SshPublicKey"
    """SshPublicKey."""
    MULTI_FACTOR = "MultiFactor"
    """MultiFactor."""


class SnowflakeAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    BASIC = "Basic"
    """Basic."""
    KEY_PAIR = "KeyPair"
    """KeyPair."""
    AAD_SERVICE_PRINCIPAL = "AADServicePrincipal"
    """AADServicePrincipal."""


class SparkAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication method used to access the Spark server."""

    ANONYMOUS = "Anonymous"
    """Anonymous."""
    USERNAME = "Username"
    """Username."""
    USERNAME_AND_PASSWORD = "UsernameAndPassword"
    """UsernameAndPassword."""
    WINDOWS_AZURE_HD_INSIGHT_SERVICE = "WindowsAzureHDInsightService"
    """WindowsAzureHDInsightService."""


class SparkConfigurationReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Spark configuration reference type."""

    SPARK_CONFIGURATION_REFERENCE = "SparkConfigurationReference"
    """SparkConfigurationReference."""


class SparkJobReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Synapse spark job reference type."""

    SPARK_JOB_DEFINITION_REFERENCE = "SparkJobDefinitionReference"
    """SparkJobDefinitionReference."""


class SparkServerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of Spark server."""

    SHARK_SERVER = "SharkServer"
    """SharkServer."""
    SHARK_SERVER2 = "SharkServer2"
    """SharkServer2."""
    SPARK_THRIFT_SERVER = "SparkThriftServer"
    """SparkThriftServer."""


class SparkThriftTransportProtocol(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The transport protocol to use in the Thrift layer."""

    BINARY = "Binary"
    """Binary."""
    SASL = "SASL"
    """SASL."""
    HTTP = "HTTP "
    """HTTP."""


class SqlAlwaysEncryptedAkvAuthType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Sql always encrypted AKV authentication type. Type: string."""

    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    MANAGED_IDENTITY = "ManagedIdentity"
    """ManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class SqlServerAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type used for authentication. Type: string."""

    SQL = "SQL"
    """SQL."""
    WINDOWS = "Windows"
    """Windows."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class SsisLogLocationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of SSIS log location."""

    FILE = "File"
    """File."""


class SsisObjectMetadataType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of SSIS object metadata."""

    FOLDER = "Folder"
    """Folder."""
    PROJECT = "Project"
    """Project."""
    PACKAGE = "Package"
    """Package."""
    ENVIRONMENT = "Environment"
    """Environment."""


class SsisPackageLocationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of SSIS package location."""

    SSISDB = "SSISDB"
    """SSISDB."""
    FILE = "File"
    """File."""
    INLINE_PACKAGE = "InlinePackage"
    """InlinePackage."""
    PACKAGE_STORE = "PackageStore"
    """PackageStore."""


class SybaseAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AuthenticationType to be used for connection."""

    BASIC = "Basic"
    """Basic."""
    WINDOWS = "Windows"
    """Windows."""


class TeamDeskAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    BASIC = "Basic"
    """Basic."""
    TOKEN = "Token"
    """Token."""


class TeradataAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """AuthenticationType to be used for connection."""

    BASIC = "Basic"
    """Basic."""
    WINDOWS = "Windows"
    """Windows."""


class TriggerReferenceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Trigger reference type."""

    TRIGGER_REFERENCE = "TriggerReference"
    """TriggerReference."""


class TriggerRunStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Trigger run status."""

    SUCCEEDED = "Succeeded"
    """Succeeded."""
    FAILED = "Failed"
    """Failed."""
    INPROGRESS = "Inprogress"
    """Inprogress."""


class TriggerRuntimeState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumerates possible state of Triggers."""

    STARTED = "Started"
    """Started."""
    STOPPED = "Stopped"
    """Stopped."""
    DISABLED = "Disabled"
    """Disabled."""


class TumblingWindowFrequency(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Enumerates possible frequency option for the tumbling window trigger."""

    MINUTE = "Minute"
    """Minute."""
    HOUR = "Hour"
    """Hour."""
    MONTH = "Month"
    """Month."""


class Type(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Linked service reference type."""

    LINKED_SERVICE_REFERENCE = "LinkedServiceReference"
    """LinkedServiceReference."""


class ValueType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of value copied from source."""

    ACTUAL = "actual"
    """actual."""
    DISPLAY = "display"
    """display."""


class VariableType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Variable type."""

    STRING = "String"
    """String."""
    BOOL = "Bool"
    """Bool."""
    ARRAY = "Array"
    """Array."""


class WarehouseAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    SYSTEM_ASSIGNED_MANAGED_IDENTITY = "SystemAssignedManagedIdentity"
    """SystemAssignedManagedIdentity."""
    USER_ASSIGNED_MANAGED_IDENTITY = "UserAssignedManagedIdentity"
    """UserAssignedManagedIdentity."""


class WebActivityMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The list of HTTP methods supported by a WebActivity."""

    GET = "GET"
    """GET."""
    POST = "POST"
    """POST."""
    PUT = "PUT"
    """PUT."""
    DELETE = "DELETE"
    """DELETE."""


class WebAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of authentication used to connect to the web table source."""

    BASIC = "Basic"
    """Basic."""
    ANONYMOUS = "Anonymous"
    """Anonymous."""
    CLIENT_CERTIFICATE = "ClientCertificate"
    """ClientCertificate."""


class WebHookActivityMethod(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The list of HTTP methods supported by a WebHook activity."""

    POST = "POST"
    """POST."""


class ZendeskAuthenticationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The authentication type to use."""

    BASIC = "Basic"
    """Basic."""
    TOKEN = "Token"
    """Token."""



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.costmanagement.aio import CostManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCostManagementGenerateCostDetailsReportOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CostManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_generate_cost_details_report_begin_get_operation_results(self, resource_group):
        response = await (
            await self.client.generate_cost_details_report.begin_get_operation_results(
                scope="str",
                operation_id="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_generate_cost_details_report_begin_create_operation(self, resource_group):
        response = await (
            await self.client.generate_cost_details_report.begin_create_operation(
                scope="str",
                parameters={
                    "billingPeriod": "str",
                    "invoiceId": "str",
                    "metric": "str",
                    "timePeriod": {"end": "str", "start": "str"},
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

"""Agent Optimization — Config loader for optimization-ready hosted agents.

One import, one call::

    from azure.ai.agentserver.optimization import load_config

    config = load_config()                          # uses .agent_configs/baseline/
    config = load_config(config_dir="my_configs")   # custom directory

Resolution order (first match wins):
    1. OPTIMIZATION_CONFIG env var   → inline JSON (used by temp agent versions)
    2. OPTIMIZATION_CANDIDATE_ID + ENDPOINT → resolver API → full config + skills
    3. Local directory (config_dir or .agent_configs/)
       → metadata.yaml + instructions.md + tools.json + skills/
    4. No config found → returns ``None``.
"""

from azure.ai.agentserver.optimization._config import load_config, load_skills_from_dir
from azure.ai.agentserver.optimization._models import (
    CandidateConfig,
    OptimizationConfig,
    Skill,
)
from azure.ai.agentserver.optimization._version import VERSION

__all__ = [
    "CandidateConfig",
    "OptimizationConfig",
    "Skill",
    "load_config",
    "load_skills_from_dir",
]
__version__ = VERSION



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.containerregistry.aio import ContainerRegistryManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestContainerRegistryManagementRegistriesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ContainerRegistryManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_get(self, resource_group):
        response = await self.client.registries.get(
            resource_group_name=resource_group.name,
            registry_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_begin_create(self, resource_group):
        response = await (
            await self.client.registries.begin_create(
                resource_group_name=resource_group.name,
                registry_name="str",
                registry={
                    "location": "str",
                    "sku": {"name": "str", "tier": "str"},
                    "id": "str",
                    "identity": {
                        "principalId": "str",
                        "tenantId": "str",
                        "type": "str",
                        "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                    },
                    "name": "str",
                    "properties": {
                        "adminUserEnabled": bool,
                        "anonymousPullEnabled": bool,
                        "autoGeneratedDomainNameLabelScope": "str",
                        "creationDate": "2020-02-20 00:00:00",
                        "dataEndpointEnabled": bool,
                        "dataEndpointHostNames": ["str"],
                        "encryption": {
                            "keyVaultProperties": {
                                "identity": "str",
                                "keyIdentifier": "str",
                                "keyRotationEnabled": bool,
                                "lastKeyRotationTimestamp": "2020-02-20 00:00:00",
                                "versionedKeyIdentifier": "str",
                            },
                            "status": "str",
                        },
                        "endpointProtocol": "str",
                        "loginServer": "str",
                        "metadataSearch": "str",
                        "networkRuleBypassAllowedForTasks": bool,
                        "networkRuleBypassOptions": "str",
                        "networkRuleSet": {"defaultAction": "str", "ipRules": [{"value": "str", "action": "str"}]},
                        "policies": {
                            "azureADAuthenticationAsArmPolicy": {"status": "str"},
                            "exportPolicy": {"status": "str"},
                            "quarantinePolicy": {"status": "str"},
                            "retentionPolicy": {"days": 0, "lastUpdatedTime": "2020-02-20 00:00:00", "status": "str"},
                            "softDeletePolicy": {
                                "lastUpdatedTime": "2020-02-20 00:00:00",
                                "retentionDays": 0,
                                "status": "str",
                            },
                            "trustPolicy": {"status": "str", "type": "str"},
                        },
                        "privateEndpointConnections": [
                            {
                                "id": "str",
                                "name": "str",
                                "properties": {
                                    "privateEndpoint": {"id": "str"},
                                    "privateLinkServiceConnectionState": {
                                        "actionsRequired": "str",
                                        "description": "str",
                                        "status": "str",
                                    },
                                    "provisioningState": "str",
                                },
                                "systemData": {
                                    "createdAt": "2020-02-20 00:00:00",
                                    "createdBy": "str",
                                    "createdByType": "str",
                                    "lastModifiedAt": "2020-02-20 00:00:00",
                                    "lastModifiedBy": "str",
                                    "lastModifiedByType": "str",
                                },
                                "type": "str",
                            }
                        ],
                        "provisioningState": "str",
                        "publicNetworkAccess": "str",
                        "regionalEndpointHostNames": ["str"],
                        "regionalEndpoints": "str",
                        "roleAssignmentMode": "str",
                        "status": {"displayStatus": "str", "message": "str", "timestamp": "2020-02-20 00:00:00"},
                        "zoneRedundancy": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "tags": {"str": "str"},
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_begin_update(self, resource_group):
        response = await (
            await self.client.registries.begin_update(
                resource_group_name=resource_group.name,
                registry_name="str",
                registry_update_parameters={
                    "identity": {
                        "principalId": "str",
                        "tenantId": "str",
                        "type": "str",
                        "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                    },
                    "properties": {
                        "adminUserEnabled": bool,
                        "anonymousPullEnabled": bool,
                        "dataEndpointEnabled": bool,
                        "encryption": {
                            "keyVaultProperties": {
                                "identity": "str",
                                "keyIdentifier": "str",
                                "keyRotationEnabled": bool,
                                "lastKeyRotationTimestamp": "2020-02-20 00:00:00",
                                "versionedKeyIdentifier": "str",
                            },
                            "status": "str",
                        },
                        "endpointProtocol": "str",
                        "metadataSearch": "str",
                        "networkRuleBypassAllowedForTasks": bool,
                        "networkRuleBypassOptions": "str",
                        "networkRuleSet": {"defaultAction": "str", "ipRules": [{"value": "str", "action": "str"}]},
                        "policies": {
                            "azureADAuthenticationAsArmPolicy": {"status": "str"},
                            "exportPolicy": {"status": "str"},
                            "quarantinePolicy": {"status": "str"},
                            "retentionPolicy": {"days": 0, "lastUpdatedTime": "2020-02-20 00:00:00", "status": "str"},
                            "softDeletePolicy": {
                                "lastUpdatedTime": "2020-02-20 00:00:00",
                                "retentionDays": 0,
                                "status": "str",
                            },
                            "trustPolicy": {"status": "str", "type": "str"},
                        },
                        "publicNetworkAccess": "str",
                        "regionalEndpoints": "str",
                        "roleAssignmentMode": "str",
                    },
                    "sku": {"name": "str", "tier": "str"},
                    "tags": {"str": "str"},
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_begin_delete(self, resource_group):
        response = await (
            await self.client.registries.begin_delete(
                resource_group_name=resource_group.name,
                registry_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_list_by_resource_group(self, resource_group):
        response = self.client.registries.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_list(self, resource_group):
        response = self.client.registries.list()
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_begin_import_image(self, resource_group):
        response = await (
            await self.client.registries.begin_import_image(
                resource_group_name=resource_group.name,
                registry_name="str",
                parameters={
                    "source": {
                        "sourceImage": "str",
                        "credentials": {"password": "str", "username": "str"},
                        "registryUri": "str",
                        "resourceId": "str",
                    },
                    "mode": "str",
                    "targetTags": ["str"],
                    "untaggedTargetRepositories": ["str"],
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_list_usages(self, resource_group):
        response = await self.client.registries.list_usages(
            resource_group_name=resource_group.name,
            registry_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_list_credentials(self, resource_group):
        response = await self.client.registries.list_credentials(
            resource_group_name=resource_group.name,
            registry_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_regenerate_credential(self, resource_group):
        response = await self.client.registries.regenerate_credential(
            resource_group_name=resource_group.name,
            registry_name="str",
            regenerate_credential_parameters={"name": "str"},
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_begin_generate_credentials(self, resource_group):
        response = await (
            await self.client.registries.begin_generate_credentials(
                resource_group_name=resource_group.name,
                registry_name="str",
                generate_credentials_parameters={"expiry": "2020-02-20 00:00:00", "name": "str", "tokenId": "str"},
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_check_name_availability(self, resource_group):
        response = await self.client.registries.check_name_availability(
            registry_name_check_request={
                "name": "str",
                "type": "str",
                "autoGeneratedDomainNameLabelScope": "str",
                "resourceGroupName": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_get_private_link_resource(self, resource_group):
        response = await self.client.registries.get_private_link_resource(
            resource_group_name=resource_group.name,
            registry_name="str",
            group_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_registries_list_private_link_resources(self, resource_group):
        response = self.client.registries.list_private_link_resources(
            resource_group_name=resource_group.name,
            registry_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.costmanagement import CostManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCostManagementExportsOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CostManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_exports_get(self, resource_group):
        response = self.client.exports.get(
            scope="str",
            export_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_exports_create_or_update(self, resource_group):
        response = self.client.exports.create_or_update(
            scope="str",
            export_name="str",
            parameters={
                "eTag": "str",
                "id": "str",
                "identity": {"type": "str", "principalId": "str", "tenantId": "str"},
                "location": "str",
                "name": "str",
                "properties": {
                    "definition": {
                        "timeframe": "str",
                        "type": "str",
                        "dataSet": {
                            "configuration": {
                                "columns": ["str"],
                                "dataVersion": "str",
                                "filters": [{"name": "str", "value": "str"}],
                            },
                            "granularity": "str",
                        },
                        "timePeriod": {"from": "2020-02-20 00:00:00", "to": "2020-02-20 00:00:00"},
                    },
                    "deliveryInfo": {
                        "destination": {
                            "container": "str",
                            "resourceId": "str",
                            "rootFolderPath": "str",
                            "sasToken": "str",
                            "storageAccount": "str",
                            "type": "str",
                        }
                    },
                    "compressionMode": "str",
                    "dataOverwriteBehavior": "str",
                    "exportDescription": "str",
                    "format": "str",
                    "nextRunTimeEstimate": "2020-02-20 00:00:00",
                    "partitionData": bool,
                    "runHistory": {
                        "value": [
                            {
                                "eTag": "str",
                                "id": "str",
                                "name": "str",
                                "properties": {
                                    "endDate": "2020-02-20 00:00:00",
                                    "error": {"code": "str", "message": "str"},
                                    "executionType": "str",
                                    "fileName": "str",
                                    "manifestFile": "str",
                                    "processingEndTime": "2020-02-20 00:00:00",
                                    "processingStartTime": "2020-02-20 00:00:00",
                                    "runSettings": {
                                        "definition": {
                                            "timeframe": "str",
                                            "type": "str",
                                            "dataSet": {
                                                "configuration": {
                                                    "columns": ["str"],
                                                    "dataVersion": "str",
                                                    "filters": [{"name": "str", "value": "str"}],
                                                },
                                                "granularity": "str",
                                            },
                                            "timePeriod": {"from": "2020-02-20 00:00:00", "to": "2020-02-20 00:00:00"},
                                        },
                                        "deliveryInfo": {
                                            "destination": {
                                                "container": "str",
                                                "resourceId": "str",
                                                "rootFolderPath": "str",
                                                "sasToken": "str",
                                                "storageAccount": "str",
                                                "type": "str",
                                            }
                                        },
                                        "compressionMode": "str",
                                        "dataOverwriteBehavior": "str",
                                        "exportDescription": "str",
                                        "format": "str",
                                        "nextRunTimeEstimate": "2020-02-20 00:00:00",
                                        "partitionData": bool,
                                        "runHistory": ...,
                                        "systemSuspensionContext": {
                                            "suspensionCode": "str",
                                            "suspensionReason": "str",
                                            "suspensionTime": "2020-02-20 00:00:00",
                                        },
                                    },
                                    "startDate": "2020-02-20 00:00:00",
                                    "status": "str",
                                    "submittedBy": "str",
                                    "submittedTime": "2020-02-20 00:00:00",
                                },
                                "type": "str",
                            }
                        ]
                    },
                    "schedule": {
                        "recurrence": "str",
                        "recurrencePeriod": {"from": "2020-02-20 00:00:00", "to": "2020-02-20 00:00:00"},
                        "status": "str",
                    },
                    "systemSuspensionContext": {
                        "suspensionCode": "str",
                        "suspensionReason": "str",
                        "suspensionTime": "2020-02-20 00:00:00",
                    },
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "type": "str",
            },
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_exports_delete(self, resource_group):
        response = self.client.exports.delete(
            scope="str",
            export_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_exports_list(self, resource_group):
        response = self.client.exports.list(
            scope="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_exports_execute(self, resource_group):
        response = self.client.exports.execute(
            scope="str",
            export_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_exports_get_execution_history(self, resource_group):
        response = self.client.exports.get_execution_history(
            scope="str",
            export_name="str",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.cloudhealth.aio import CloudHealthMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestCloudHealthMgmtDiscoveryRulesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(CloudHealthMgmtClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discovery_rules_get(self, resource_group):
        response = await self.client.discovery_rules.get(
            resource_group_name=resource_group.name,
            health_model_name="str",
            discovery_rule_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discovery_rules_begin_create_or_update(self, resource_group):
        response = await (
            await self.client.discovery_rules.begin_create_or_update(
                resource_group_name=resource_group.name,
                health_model_name="str",
                discovery_rule_name="str",
                resource={
                    "id": "str",
                    "name": "str",
                    "properties": {
                        "addRecommendedSignals": "str",
                        "authenticationSetting": "str",
                        "discoverRelationships": "str",
                        "entityName": "str",
                        "specification": "discovery_rule_specification",
                        "displayName": "str",
                        "error": {"message": "str", "context": ["str"]},
                        "provisioningState": "str",
                    },
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "type": "str",
                },
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discovery_rules_begin_delete(self, resource_group):
        response = await (
            await self.client.discovery_rules.begin_delete(
                resource_group_name=resource_group.name,
                health_model_name="str",
                discovery_rule_name="str",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_discovery_rules_list_by_health_model(self, resource_group):
        response = self.client.discovery_rules.list_by_health_model(
            resource_group_name=resource_group.name,
            health_model_name="str",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.datamigration.aio import DataMigrationManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDataMigrationManagementDatabaseMigrationsMongoToCosmosDbvCoreMongoOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DataMigrationManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_database_migrations_mongo_to_cosmos_dbv_core_mongo_get(self, resource_group):
        response = await self.client.database_migrations_mongo_to_cosmos_dbv_core_mongo.get(
            resource_group_name=resource_group.name,
            target_resource_name="str",
            migration_name="str",
            api_version="2025-06-30",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_database_migrations_mongo_to_cosmos_dbv_core_mongo_begin_create(self, resource_group):
        response = await (
            await self.client.database_migrations_mongo_to_cosmos_dbv_core_mongo.begin_create(
                resource_group_name=resource_group.name,
                target_resource_name="str",
                migration_name="str",
                parameters={
                    "collectionList": [
                        {
                            "migrationProgressDetails": {
                                "durationInSeconds": 0,
                                "migrationError": "str",
                                "migrationStatus": "str",
                                "processedDocumentCount": 0,
                                "sourceDocumentCount": 0,
                            },
                            "sourceCollection": "str",
                            "sourceDatabase": "str",
                            "targetCollection": "str",
                            "targetDatabase": "str",
                        }
                    ],
                    "endedOn": "2020-02-20 00:00:00",
                    "id": "str",
                    "migrationFailureError": {"code": "str", "message": "str"},
                    "migrationOperationId": "str",
                    "migrationService": "str",
                    "migrationStatus": "str",
                    "name": "str",
                    "provisioningError": "str",
                    "provisioningState": "str",
                    "scope": "str",
                    "sourceMongoConnection": {
                        "connectionString": "str",
                        "host": "str",
                        "password": "str",
                        "port": 0,
                        "useSsl": bool,
                        "userName": "str",
                    },
                    "startedOn": "2020-02-20 00:00:00",
                    "systemData": {
                        "createdAt": "2020-02-20 00:00:00",
                        "createdBy": "str",
                        "createdByType": "str",
                        "lastModifiedAt": "2020-02-20 00:00:00",
                        "lastModifiedBy": "str",
                        "lastModifiedByType": "str",
                    },
                    "targetMongoConnection": {
                        "connectionString": "str",
                        "host": "str",
                        "password": "str",
                        "port": 0,
                        "useSsl": bool,
                        "userName": "str",
                    },
                    "type": "str",
                },
                api_version="2025-06-30",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_database_migrations_mongo_to_cosmos_dbv_core_mongo_begin_delete(self, resource_group):
        response = await (
            await self.client.database_migrations_mongo_to_cosmos_dbv_core_mongo.begin_delete(
                resource_group_name=resource_group.name,
                target_resource_name="str",
                migration_name="str",
                api_version="2025-06-30",
            )
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_database_migrations_mongo_to_cosmos_dbv_core_mongo_get_for_scope(self, resource_group):
        response = self.client.database_migrations_mongo_to_cosmos_dbv_core_mongo.get_for_scope(
            resource_group_name=resource_group.name,
            target_resource_name="str",
            api_version="2025-06-30",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
from typing import Any, Callable, Dict, IO, Iterator, Optional, TypeVar, Union, cast, overload

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.polling import LROPoller, NoPolling, PollingMethod
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.arm_polling import ARMPolling

from .. import models as _models
from .._configuration import DataMigrationManagementClientConfiguration
from .._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_get_request(
    resource_group_name: str,
    sql_db_instance_name: str,
    target_db_name: str,
    subscription_id: str,
    *,
    migration_operation_id: Optional[str] = None,
    expand: Optional[str] = None,
    **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-06-30"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/servers/{sqlDbInstanceName}/providers/Microsoft.DataMigration/databaseMigrations/{targetDbName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "sqlDbInstanceName": _SERIALIZER.url("sql_db_instance_name", sql_db_instance_name, "str"),
        "targetDbName": _SERIALIZER.url("target_db_name", target_db_name, "str"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    if migration_operation_id is not None:
        _params["migrationOperationId"] = _SERIALIZER.query("migration_operation_id", migration_operation_id, "str")
    if expand is not None:
        _params["$expand"] = _SERIALIZER.query("expand", expand, "str")
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_or_update_request(
    resource_group_name: str, sql_db_instance_name: str, target_db_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-06-30"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/servers/{sqlDbInstanceName}/providers/Microsoft.DataMigration/databaseMigrations/{targetDbName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "sqlDbInstanceName": _SERIALIZER.url("sql_db_instance_name", sql_db_instance_name, "str"),
        "targetDbName": _SERIALIZER.url("target_db_name", target_db_name, "str"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_delete_request(
    resource_group_name: str,
    sql_db_instance_name: str,
    target_db_name: str,
    subscription_id: str,
    *,
    force: Optional[bool] = None,
    **kwargs: Any
) -> HttpRequest:
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-06-30"))
    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/servers/{sqlDbInstanceName}/providers/Microsoft.DataMigration/databaseMigrations/{targetDbName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "sqlDbInstanceName": _SERIALIZER.url("sql_db_instance_name", sql_db_instance_name, "str"),
        "targetDbName": _SERIALIZER.url("target_db_name", target_db_name, "str"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    if force is not None:
        _params["force"] = _SERIALIZER.query("force", force, "bool")
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, **kwargs)


def build_cancel_request(
    resource_group_name: str, sql_db_instance_name: str, target_db_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-06-30"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/servers/{sqlDbInstanceName}/providers/Microsoft.DataMigration/databaseMigrations/{targetDbName}/cancel",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "sqlDbInstanceName": _SERIALIZER.url("sql_db_instance_name", sql_db_instance_name, "str"),
        "targetDbName": _SERIALIZER.url("target_db_name", target_db_name, "str"),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_retry_request(
    resource_group_name: str, sql_db_instance_name: str, target_db_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-06-30"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Sql/servers/{sqlDbInstanceName}/providers/Microsoft.DataMigration/databaseMigrations/{targetDbName}/retry",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url("resource_group_name", resource_group_name, "str"),
        "sqlDbInstanceName": _SERIALIZER.url(
            "sql_db_instance_name", sql_db_instance_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_.-]{0,127}$"
        ),
        "targetDbName": _SERIALIZER.url(
            "target_db_name", target_db_name, "str", pattern=r"^[a-zA-Z0-9][a-zA-Z0-9_.-]{0,127}$"
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


class DatabaseMigrationsSqlDbOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.datamigration.DataMigrationManagementClient`'s
        :attr:`database_migrations_sql_db` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: DataMigrationManagementClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def get(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        migration_operation_id: Optional[str] = None,
        expand: Optional[str] = None,
        **kwargs: Any
    ) -> _models.DatabaseMigrationSqlDb:
        """Retrieve the Database Migration resource.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param migration_operation_id: Optional migration operation ID. If this is provided, then
         details of migration operation for that ID are retrieved. If not provided (default), then
         details related to most recent or current operation are retrieved. Default value is None.
        :type migration_operation_id: str
        :param expand: Complete migration details be included in the response. Default value is None.
        :type expand: str
        :return: DatabaseMigrationSqlDb or the result of cls(response)
        :rtype: ~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.DatabaseMigrationSqlDb] = kwargs.pop("cls", None)

        _request = build_get_request(
            resource_group_name=resource_group_name,
            sql_db_instance_name=sql_db_instance_name,
            target_db_name=target_db_name,
            subscription_id=self._config.subscription_id,
            migration_operation_id=migration_operation_id,
            expand=expand,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize("DatabaseMigrationSqlDb", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    def _create_or_update_initial(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: Union[_models.DatabaseMigrationSqlDb, IO[bytes]],
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "DatabaseMigrationSqlDb")

        _request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            sql_db_instance_name=sql_db_instance_name,
            target_db_name=target_db_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def begin_create_or_update(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: _models.DatabaseMigrationSqlDb,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.DatabaseMigrationSqlDb]:
        """Create or Update Database Migration resource.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param parameters: Details of Sql Db migration resource. Required.
        :type parameters: ~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either DatabaseMigrationSqlDb or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_create_or_update(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.DatabaseMigrationSqlDb]:
        """Create or Update Database Migration resource.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param parameters: Details of Sql Db migration resource. Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either DatabaseMigrationSqlDb or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_create_or_update(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: Union[_models.DatabaseMigrationSqlDb, IO[bytes]],
        **kwargs: Any
    ) -> LROPoller[_models.DatabaseMigrationSqlDb]:
        """Create or Update Database Migration resource.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param parameters: Details of Sql Db migration resource. Is either a DatabaseMigrationSqlDb
         type or a IO[bytes] type. Required.
        :type parameters: ~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb or IO[bytes]
        :return: An instance of LROPoller that returns either DatabaseMigrationSqlDb or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.DatabaseMigrationSqlDb] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._create_or_update_initial(
                resource_group_name=resource_group_name,
                sql_db_instance_name=sql_db_instance_name,
                target_db_name=target_db_name,
                parameters=parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("DatabaseMigrationSqlDb", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[_models.DatabaseMigrationSqlDb].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[_models.DatabaseMigrationSqlDb](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )

    def _delete_initial(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        force: Optional[bool] = None,
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            sql_db_instance_name=sql_db_instance_name,
            target_db_name=target_db_name,
            subscription_id=self._config.subscription_id,
            force=force,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def begin_delete(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        force: Optional[bool] = None,
        **kwargs: Any
    ) -> LROPoller[None]:
        """Delete Database Migration resource.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param force: Optional force delete boolean. If this is provided as true, migration will be
         deleted even if active. Default value is None.
        :type force: bool
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._delete_initial(
                resource_group_name=resource_group_name,
                sql_db_instance_name=sql_db_instance_name,
                target_db_name=target_db_name,
                force=force,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    def _cancel_initial(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: Union[_models.MigrationOperationInput, IO[bytes]],
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "MigrationOperationInput")

        _request = build_cancel_request(
            resource_group_name=resource_group_name,
            sql_db_instance_name=sql_db_instance_name,
            target_db_name=target_db_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def begin_cancel(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: _models.MigrationOperationInput,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[None]:
        """Stop on going migration for the database.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param parameters: Required migration operation ID for which cancel will be initiated.
         Required.
        :type parameters: ~azure.mgmt.datamigration.models.MigrationOperationInput
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_cancel(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[None]:
        """Stop on going migration for the database.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param parameters: Required migration operation ID for which cancel will be initiated.
         Required.
        :type parameters: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_cancel(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        parameters: Union[_models.MigrationOperationInput, IO[bytes]],
        **kwargs: Any
    ) -> LROPoller[None]:
        """Stop on going migration for the database.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param parameters: Required migration operation ID for which cancel will be initiated. Is
         either a MigrationOperationInput type or a IO[bytes] type. Required.
        :type parameters: ~azure.mgmt.datamigration.models.MigrationOperationInput or IO[bytes]
        :return: An instance of LROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.LROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._cancel_initial(
                resource_group_name=resource_group_name,
                sql_db_instance_name=sql_db_instance_name,
                target_db_name=target_db_name,
                parameters=parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})  # type: ignore

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[None].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[None](self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    def _retry_initial(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        migration_operation_input: Union[_models.MigrationOperationInput, IO[bytes]],
        **kwargs: Any
    ) -> Iterator[bytes]:
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(migration_operation_input, (IOBase, bytes)):
            _content = migration_operation_input
        else:
            _json = self._serialize.body(migration_operation_input, "MigrationOperationInput")

        _request = build_retry_request(
            resource_group_name=resource_group_name,
            sql_db_instance_name=sql_db_instance_name,
            target_db_name=target_db_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _decompress = kwargs.pop("decompress", True)
        _stream = True
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            try:
                response.read()  # Load the body in memory and close the socket
            except (StreamConsumedError, StreamClosedError):
                pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = response.stream_download(self._client._pipeline, decompress=_decompress)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def begin_retry(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        migration_operation_input: _models.MigrationOperationInput,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.DatabaseMigrationSqlDb]:
        """Retry on going migration for the database.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param migration_operation_input: Required migration operation ID for which retry will be
         initiated. Required.
        :type migration_operation_input: ~azure.mgmt.datamigration.models.MigrationOperationInput
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either DatabaseMigrationSqlDb or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def begin_retry(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        migration_operation_input: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> LROPoller[_models.DatabaseMigrationSqlDb]:
        """Retry on going migration for the database.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param migration_operation_input: Required migration operation ID for which retry will be
         initiated. Required.
        :type migration_operation_input: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: An instance of LROPoller that returns either DatabaseMigrationSqlDb or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def begin_retry(
        self,
        resource_group_name: str,
        sql_db_instance_name: str,
        target_db_name: str,
        migration_operation_input: Union[_models.MigrationOperationInput, IO[bytes]],
        **kwargs: Any
    ) -> LROPoller[_models.DatabaseMigrationSqlDb]:
        """Retry on going migration for the database.

        :param resource_group_name: Name of the resource group that contains the resource. You can
         obtain this value from the Azure Resource Manager API or the portal. Required.
        :type resource_group_name: str
        :param sql_db_instance_name: Required.
        :type sql_db_instance_name: str
        :param target_db_name: The name of the target database. Required.
        :type target_db_name: str
        :param migration_operation_input: Required migration operation ID for which retry will be
         initiated. Is either a MigrationOperationInput type or a IO[bytes] type. Required.
        :type migration_operation_input: ~azure.mgmt.datamigration.models.MigrationOperationInput or
         IO[bytes]
        :return: An instance of LROPoller that returns either DatabaseMigrationSqlDb or the result of
         cls(response)
        :rtype: ~azure.core.polling.LROPoller[~azure.mgmt.datamigration.models.DatabaseMigrationSqlDb]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.DatabaseMigrationSqlDb] = kwargs.pop("cls", None)
        polling: Union[bool, PollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = self._retry_initial(
                resource_group_name=resource_group_name,
                sql_db_instance_name=sql_db_instance_name,
                target_db_name=target_db_name,
                migration_operation_input=migration_operation_input,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
            raw_result.http_response.read()  # type: ignore
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("DatabaseMigrationSqlDb", pipeline_response.http_response)
            if cls:
                return cls(pipeline_response, deserialized, {})  # type: ignore
            return deserialized

        if polling is True:
            polling_method: PollingMethod = cast(PollingMethod, ARMPolling(lro_delay, **kwargs))
        elif polling is False:
            polling_method = cast(PollingMethod, NoPolling())
        else:
            polling_method = polling
        if cont_token:
            return LROPoller[_models.DatabaseMigrationSqlDb].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return LROPoller[_models.DatabaseMigrationSqlDb](
            self._client, raw_result, get_long_running_output, polling_method  # type: ignore
        )



# The MIT License (MIT)
# Copyright (c) 2025 Microsoft Corporation

"""Class for global endpoint manager for per partition automatic failover. This class inherits the circuit breaker
endpoint manager, since enabling per partition automatic failover also enables the circuit breaker logic.
"""
import logging
import threading
import os

from typing import TYPE_CHECKING, Optional

from azure.cosmos.http_constants import ResourceType
from azure.cosmos._constants import _Constants as Constants
from azure.cosmos.aio._global_partition_endpoint_manager_circuit_breaker_async import \
    _GlobalPartitionEndpointManagerForCircuitBreakerAsync
from azure.cosmos.documents import _OperationType
from azure.cosmos._partition_health_tracker import _PPAFPartitionThresholdsTracker
from azure.cosmos._request_object import RequestObject
from azure.cosmos._routing.routing_range import PartitionKeyRangeWrapper

if TYPE_CHECKING:
    from azure.cosmos.aio._cosmos_client_connection_async import CosmosClientConnection
    from azure.cosmos._location_cache import RegionalRoutingContext

logger = logging.getLogger("azure.cosmos._GlobalPartitionEndpointManagerForPerPartitionAutomaticFailover")

# pylint: disable=name-too-long, protected-access, too-many-nested-blocks
#cspell:ignore PPAF, ppaf, ppcb

class PartitionLevelFailoverInfo:
    """
    Holds information about the partition level regional failover.
    Used to track the partition key range and the regions where it is available.
    """
    def __init__(self) -> None:
        self.unavailable_regional_endpoints: dict[str, "RegionalRoutingContext"] = {}
        self._lock = threading.Lock()
        self.current_region: Optional[str] = None

    def try_move_to_next_location(
            self,
            available_account_regional_endpoints: dict[str, "RegionalRoutingContext"],
            endpoint_region: str,
            request: RequestObject) -> bool:
        """
        Tries to move to the next available regional endpoint for the partition key range.
        :param Dict[str, RegionalRoutingContext] available_account_regional_endpoints: The available regional endpoints
        :param str endpoint_region: The current regional endpoint
        :param RequestObject request: The request object containing the routing context.
        :return: True if the move was successful, False otherwise.
        :rtype: bool
        """
        with self._lock:
            if endpoint_region != self.current_region and self.current_region is not None:
                regional_endpoint = available_account_regional_endpoints[self.current_region].primary_endpoint
                request.route_to_location(regional_endpoint)
                return True

            for regional_endpoint in available_account_regional_endpoints:
                if regional_endpoint == self.current_region:
                    continue

                if regional_endpoint in self.unavailable_regional_endpoints:
                    continue

                self.current_region = regional_endpoint
                logger.warning("PPAF - Moving to next available regional endpoint: %s", self.current_region)
                regional_endpoint = available_account_regional_endpoints[self.current_region].primary_endpoint
                request.route_to_location(regional_endpoint)
                return True

            return False

class _GlobalPartitionEndpointManagerForPerPartitionAutomaticFailoverAsync(
    _GlobalPartitionEndpointManagerForCircuitBreakerAsync):
    """
    This internal class implements the logic for partition endpoint management for
    geo-replicated database accounts.
    """
    def __init__(self, client: "CosmosClientConnection") -> None:
        super(_GlobalPartitionEndpointManagerForPerPartitionAutomaticFailoverAsync, self).__init__(client)
        self.partition_range_to_failover_info: dict[PartitionKeyRangeWrapper, PartitionLevelFailoverInfo] = {}
        self.ppaf_thresholds_tracker = _PPAFPartitionThresholdsTracker()
        self._threshold_lock = threading.Lock()

    def is_per_partition_automatic_failover_enabled(self) -> bool:
        if not self._database_account_cache or not self._database_account_cache._EnablePerPartitionFailoverBehavior:
            return False
        return True

    def is_per_partition_automatic_failover_applicable(self, request: RequestObject) -> bool:
        if not self.is_per_partition_automatic_failover_enabled():
            return False

        if not request:
            return False

        if (self.location_cache.can_use_multiple_write_locations_for_request(request)
                or _OperationType.IsReadOnlyOperation(request.operation_type)):
            return False

        # if we have at most one region available in the account, we cannot do per partition automatic failover
        available_regions = self.location_cache.account_read_regional_routing_contexts_by_location
        if len(available_regions) <= 1:
            return False

        # if the request is not a non-query plan document request
        # or if the request is not executing a stored procedure, return False
        if (request.resource_type != ResourceType.Document and
                request.operation_type != _OperationType.ExecuteJavaScript):
            return False

        return True

    def try_ppaf_failover_threshold(
            self,
            pk_range_wrapper: "PartitionKeyRangeWrapper",
            request: "RequestObject"):
        """Verifies whether the per-partition failover threshold has been reached for consecutive errors. If so,
        it marks the current region as unavailable for the given partition key range, and moves to the next available
        region for the request.

        :param PartitionKeyRangeWrapper pk_range_wrapper: The wrapper containing the partition key range information
            for the request.
        :param RequestObject request: The request object containing the routing context.
        :returns: None
        """
        # If PPAF is enabled, we track consecutive failures for certain exceptions, and only fail over at a partition
        # level after the threshold is reached
        if request and self.is_per_partition_automatic_failover_applicable(request):
            if (self.ppaf_thresholds_tracker.get_pk_failures(pk_range_wrapper)
                    >= int(os.environ.get(Constants.TIMEOUT_ERROR_THRESHOLD_PPAF,
                                          Constants.TIMEOUT_ERROR_THRESHOLD_PPAF_DEFAULT))):
                # If the PPAF threshold is reached, we reset the count and mark the endpoint unavailable
                # Once we mark the endpoint unavailable, the PPAF endpoint manager will try to move to the next
                # available region for the partition key range
                with self._threshold_lock:
                    # Check for count again, since a previous request may have now reset the count
                    if (self.ppaf_thresholds_tracker.get_pk_failures(pk_range_wrapper)
                            >= int(os.environ.get(Constants.TIMEOUT_ERROR_THRESHOLD_PPAF,
                                                  Constants.TIMEOUT_ERROR_THRESHOLD_PPAF_DEFAULT))):
                        self.ppaf_thresholds_tracker.clear_pk_failures(pk_range_wrapper)
                        partition_level_info = self.partition_range_to_failover_info[pk_range_wrapper]
                        location = self.location_cache.get_location_from_endpoint(
                            str(request.location_endpoint_to_route))
                        logger.warning("PPAF - Failover threshold reached for partition key range: %s for region: %s", #pylint: disable=line-too-long
                                       pk_range_wrapper, location)
                        regional_context = (self.location_cache.
                                            account_read_regional_routing_contexts_by_location.
                                            get(location).primary_endpoint)
                        partition_level_info.unavailable_regional_endpoints[location] = regional_context

    def resolve_service_endpoint_for_partition(
            self,
            request: RequestObject,
            pk_range_wrapper: Optional[PartitionKeyRangeWrapper]
    ) -> str:
        """Resolves the endpoint to be used for the request. In a PPAF-enabled account, this method checks whether
        the partition key range has any unavailable regions, and if so, it tries to move to the next available region.
        If all regions are unavailable, it invalidates the cache and starts once again from the main write region in the
        account configurations.

        :param PartitionKeyRangeWrapper pk_range_wrapper: The wrapper containing the partition key range information
            for the request.
        :param RequestObject request: The request object containing the routing context.
        :returns: The regional endpoint to be used for the request.
        :rtype: str
        """
        if self.is_per_partition_automatic_failover_applicable(request) and pk_range_wrapper:
            # If per partition automatic failover is applicable, we check partition unavailability
            if pk_range_wrapper in self.partition_range_to_failover_info:
                partition_failover_info = self.partition_range_to_failover_info[pk_range_wrapper]
                if request.location_endpoint_to_route is not None:
                    endpoint_region = self.location_cache.get_location_from_endpoint(request.location_endpoint_to_route)
                    if endpoint_region in partition_failover_info.unavailable_regional_endpoints:
                        available_account_regional_endpoints = self.location_cache.account_read_regional_routing_contexts_by_location #pylint: disable=line-too-long
                        if (partition_failover_info.current_region is not None and
                                endpoint_region != partition_failover_info.current_region):
                            # this request has not yet seen there's an available region being used for this partition
                            regional_endpoint = available_account_regional_endpoints[
                                partition_failover_info.current_region].primary_endpoint
                            request.route_to_location(regional_endpoint)
                        else:
                            if (len(self.location_cache.account_read_regional_routing_contexts_by_location) ==
                                    len(partition_failover_info.unavailable_regional_endpoints)):
                                # If no other region is available, we invalidate the cache and start once again
                                # from our main write region in the account configurations
                                logger.warning("All available regions for partition %s are unavailable."
                                               " Refreshing cache.", pk_range_wrapper)
                                self.partition_range_to_failover_info[pk_range_wrapper] = PartitionLevelFailoverInfo()
                                request.clear_route_to_location()
                            else:
                                # If the current region is unavailable, we try to move to the next available region
                                partition_failover_info.try_move_to_next_location(
                                    self.location_cache.account_read_regional_routing_contexts_by_location,
                                    endpoint_region,
                                    request)
                    else:
                        # Update the current regional endpoint to whatever the request is routing to
                        partition_failover_info.current_region = endpoint_region
            else:
                partition_failover_info = PartitionLevelFailoverInfo()
                endpoint_region = self.location_cache.get_location_from_endpoint(
                    request.location_endpoint_to_route)
                partition_failover_info.current_region = endpoint_region
                self.partition_range_to_failover_info[pk_range_wrapper] = partition_failover_info
        return self._resolve_service_endpoint_for_partition_circuit_breaker(request, pk_range_wrapper)

    async def record_failure(self,
                             request: RequestObject,
                             pk_range_wrapper: Optional[PartitionKeyRangeWrapper] = None) -> None:
        """Records a failure for the given partition key range and request.
        :param RequestObject request: The request object containing the routing context.
        :param PartitionKeyRangeWrapper pk_range_wrapper: The wrapper containing the partition key range information
            for the request.
        :return: None
        """
        if self.is_per_partition_automatic_failover_applicable(request):
            if pk_range_wrapper is None:
                pk_range_wrapper = await self.create_pk_range_wrapper(request)
            if pk_range_wrapper:
                self.ppaf_thresholds_tracker.add_failure(pk_range_wrapper)
        else:
            await self.record_ppcb_failure(request, pk_range_wrapper)

    async def record_success(self,
                             request: RequestObject,
                             pk_range_wrapper: Optional[PartitionKeyRangeWrapper] = None) -> None:
        """Records a success for the given partition key range and request, effectively clearing the failure count.
        :param RequestObject request: The request object containing the routing context.
        :param PartitionKeyRangeWrapper pk_range_wrapper: The wrapper containing the partition key range information
            for the request.
        :return: None
        """
        if self.is_per_partition_automatic_failover_applicable(request):
            if pk_range_wrapper is None:
                pk_range_wrapper = await self.create_pk_range_wrapper(request)
            if pk_range_wrapper:
                self.ppaf_thresholds_tracker.clear_pk_failures(pk_range_wrapper)
        else:
            await self.record_ppcb_success(request, pk_range_wrapper)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.dashboard import DashboardManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDashboardManagementGrafanaOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DashboardManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_get(self, resource_group):
        response = self.client.grafana.get(
            resource_group_name=resource_group.name,
            workspace_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_begin_create(self, resource_group):
        response = self.client.grafana.begin_create(
            resource_group_name=resource_group.name,
            workspace_name="str",
            request_body_parameters={
                "id": "str",
                "identity": {
                    "type": "str",
                    "principalId": "str",
                    "tenantId": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "location": "str",
                "name": "str",
                "properties": {
                    "apiKey": "str",
                    "autoGeneratedDomainNameLabelScope": "str",
                    "creatorCanAdmin": "str",
                    "deterministicOutboundIP": "str",
                    "endpoint": "str",
                    "enterpriseConfigurations": {"marketplaceAutoRenew": "str", "marketplacePlanId": "str"},
                    "grafanaConfigurations": {
                        "security": {"csrfAlwaysCheck": bool},
                        "smtp": {
                            "enabled": bool,
                            "fromAddress": "str",
                            "fromName": "str",
                            "host": "str",
                            "password": "str",
                            "skipVerify": bool,
                            "startTLSPolicy": "str",
                            "user": "str",
                        },
                        "snapshots": {"externalEnabled": bool},
                        "unifiedAlertingScreenshots": {"captureEnabled": bool},
                        "users": {"editorsCanAdmin": bool, "viewersCanEdit": bool},
                    },
                    "grafanaIntegrations": {
                        "azureMonitorWorkspaceIntegrations": [{"azureMonitorWorkspaceResourceId": "str"}]
                    },
                    "grafanaMajorVersion": "str",
                    "grafanaPlugins": {"str": {"pluginId": "str"}},
                    "grafanaVersion": "str",
                    "outboundIPs": ["str"],
                    "privateEndpointConnections": [
                        {
                            "id": "str",
                            "name": "str",
                            "properties": {
                                "privateLinkServiceConnectionState": {
                                    "actionsRequired": "str",
                                    "description": "str",
                                    "status": "str",
                                },
                                "groupIds": ["str"],
                                "privateEndpoint": {"id": "str"},
                                "provisioningState": "str",
                            },
                            "systemData": {
                                "createdAt": "2020-02-20 00:00:00",
                                "createdBy": "str",
                                "createdByType": "str",
                                "lastModifiedAt": "2020-02-20 00:00:00",
                                "lastModifiedBy": "str",
                                "lastModifiedByType": "str",
                            },
                            "type": "str",
                        }
                    ],
                    "provisioningState": "str",
                    "publicNetworkAccess": "str",
                    "zoneRedundancy": "str",
                },
                "sku": {"name": "str", "size": "str"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_begin_update(self, resource_group):
        response = self.client.grafana.begin_update(
            resource_group_name=resource_group.name,
            workspace_name="str",
            request_body_parameters={
                "identity": {
                    "type": "str",
                    "principalId": "str",
                    "tenantId": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "properties": {
                    "apiKey": "str",
                    "creatorCanAdmin": "str",
                    "deterministicOutboundIP": "str",
                    "enterpriseConfigurations": {"marketplaceAutoRenew": "str", "marketplacePlanId": "str"},
                    "grafanaConfigurations": {
                        "security": {"csrfAlwaysCheck": bool},
                        "smtp": {
                            "enabled": bool,
                            "fromAddress": "str",
                            "fromName": "str",
                            "host": "str",
                            "password": "str",
                            "skipVerify": bool,
                            "startTLSPolicy": "str",
                            "user": "str",
                        },
                        "snapshots": {"externalEnabled": bool},
                        "unifiedAlertingScreenshots": {"captureEnabled": bool},
                        "users": {"editorsCanAdmin": bool, "viewersCanEdit": bool},
                    },
                    "grafanaIntegrations": {
                        "azureMonitorWorkspaceIntegrations": [{"azureMonitorWorkspaceResourceId": "str"}]
                    },
                    "grafanaMajorVersion": "str",
                    "grafanaPlugins": {"str": {"pluginId": "str"}},
                    "publicNetworkAccess": "str",
                    "zoneRedundancy": "str",
                },
                "sku": {"name": "str", "size": "str"},
                "tags": {"str": "str"},
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_begin_delete(self, resource_group):
        response = self.client.grafana.begin_delete(
            resource_group_name=resource_group.name,
            workspace_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_list_by_resource_group(self, resource_group):
        response = self.client.grafana.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_list(self, resource_group):
        response = self.client.grafana.list()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_check_enterprise_details(self, resource_group):
        response = self.client.grafana.check_enterprise_details(
            resource_group_name=resource_group.name,
            workspace_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_grafana_fetch_available_plugins(self, resource_group):
        response = self.client.grafana.fetch_available_plugins(
            resource_group_name=resource_group.name,
            workspace_name="str",
        )

        # please add some check logic here by yourself
        # ...
